clc
close all
clear all

%% Copyright (c) 2018 by the authors of "Quantifying tongue tip shape in apical and laminal /s/: contributions of palate shape"

% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

%% code structure

% the code includes 3 parts: 
% (1) extraction of raw data
% (2) calculation of metrics for differentiation of apical and laminal /s/
% (3) individual result output
 
% There are 3 methods to calculate the output metric, which are explained
% in the paper. Use variable called metric_type to switch between metrics.

% NOTE: images and statistical analysis are detailed in the paper

% NOTE: there are other features baked into the code, which are not used in
% this demo, but can be invoked with modification.

%% extract data

% read xls file
[N,T] = xlsread('CurvatureInput_8Points.xlsx');

% discard header
T(1,:) = [];

% pad to match n
T = [T; repmat(T(2,:),[size(N,1)-size(T,1) 1])];

% discard nans
T(isnan(N(:,1)),:) = [];
N(isnan(N(:,1)),:) = [];

% get textrow
Ip = ~strcmp(T(:,1),'');
Ip = find(Ip);

% number of points
In = N(1<abs(diff(N(:,1))),1);
In(numel(Ip)) = N(end,1);

% IDs 
IDs = T(Ip,:);

% get data
for cnt_p = 1:numel(Ip)
    
    Pdata{cnt_p,1}.id = cell2mat(IDs(cnt_p,1));
    Pdata{cnt_p,1}.type = cell2mat(IDs(cnt_p,6));
    
    P_coordinates = N(Ip(cnt_p):Ip(cnt_p)+In(cnt_p)-1,:);
    
    P_n = P_coordinates(:,1);
    P_x = P_coordinates(:,2);
    P_y = P_coordinates(:,3);
    P_z = P_coordinates(:,4);
    P_t = P_coordinates(:,5);
    
    % this is here to check coordinate sign change
    Pdata{cnt_p,1}.points = [P_n P_x P_y P_z P_t];
    
end

%% get curvature values
% set to 'pt_order' for pictures with points, and 'curvature' for pictures
% with curvature values
folder = 'curvature'; 
fid = fopen([folder '/' '1_curvatures.txt'],'w');

bulma = cell(size(Pdata,1),1);

for cnt_p = 1:size(Pdata,1)

    % *** METHOD TYPE
    % 'MC', 'ALC', 'NALC', or 'NALCi'
    metric_type = 'NALCi';
    
    % *** POINTWISE  CALCULATIONS
    
    % set fitting options
    options.spline = 'interpolating'; % 
    options.mode = 'fit';
    options.flag = 0;  % 3 = pt number, 4 = curvature, 0 = no plot
    
    % get points
    W = Pdata{cnt_p}.points;
    
    % calculate local curvatures 
    interpolation_pts = 0;
    if strcmp(metric_type,'NALCi')
        interpolation_pts = 10;
    end 
    [X,K,~,~,N,~] = curvatureCalc(W(:,2:4),interpolation_pts,options); 
    
    % fit overall cicle for global radius
    [r,C,n] = fitcircle3D(W(:,2:4));

    % add sign to curvature
    for cnt_v = 1:size(K,1)
        % find orientation with respect to direction to center of the overall
        % circle
        Vp = X(cnt_v,:);
        Vc = C-Vp; 
        Vn = N(cnt_v,:);
        if 90<acosd(dot(Vc/norm(Vc),Vn))
            K(cnt_v) = -K(cnt_v);
        end
    end
    
    % *** METRIC CALCULATIONS
    
    % 'MC', 'ALC', 'NALC', or 'NALCi'
    switch  metric_type 
        
        case 'MC'
            % minimum curvature (MC)
            TMP = K;
            metric(cnt_p,1) = min(TMP([6 5 4])); % apical points
    
        case 'ALC'
            % averaged maximum curvature (ALC)
            TMP = K(2:end-1);
            TMP = sort(TMP,'descend');
            metric(cnt_p,1) = mean(TMP(1:2));
    
        case 'NALC'
            % normalized averaged maximum curvature (NALC)
            TMP = K(2:end-1);
            TMP = sort(TMP,'descend');
            metric(cnt_p,1) = (1/mean(TMP(1:2)))/r;
    
        case 'NALCi'
            % normalized averaged maximum curvature interpolated (NALCi)
            TMP = K(2:end-1);
            TMP = sort(TMP,'descend');
            metric(cnt_p,1) = (1/mean(TMP(1:20)))/r; 
            
        otherwise
            error(['metric: ' metric_type ' not supported'])
    end


end

%% output

disp([metric_type ' results (Control 1 and Contol 2 denote /s/ type):'])

for cnt_p = 1:size(Pdata,1) 
    
    disp([Pdata{cnt_p,1}.type ' : ' num2str(metric(cnt_p)) ])
end



